/*  -*- c -*-
   $RCSfile: canvasdot.c,v $
   $Revision: 1.8 $
   $Author: reida $
   $Date: 2004/08/26 17:33:00 $  */

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modifed
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

/* Custom canvas item, draws a circle on the canvas whose 
   radius can be specified in pixels.  For some strange reason,
   the rectangle/ellipse functions of the stock canvas do not do this. 
   This item is a subclass of the GtkCanvasRE. */

#include "canvasdot.h"
#include <libart_lgpl/art_vpath.h>
#include <stdio.h>

#define N_DOT_POINTS 36

static GtkCanvasREClass *parent_class;


/* This function duplicates the "gtk_canvas_gen_ellipse"
   functionality, which cannot be used here because "N_PTS" is
   #define'd in the .c file and not the .h file, and so is not shared,
   and so we can not make promises about the array size. */
void private_circle_gen(ArtVpath *vpath, double x0, double y0, double size) 
{
  int i;
  double cos(),sin();

  for(i=0;i<=N_DOT_POINTS;i++) {
    double th;
    th= (2*M_PI*i)/N_DOT_POINTS;
    vpath[i].code = (i==0 ? ART_MOVETO : ART_LINETO);
    vpath[i].x = x0 + size*cos(th);
    vpath[i].y = y0 + size*sin(th);
  }
  vpath[N_DOT_POINTS+1].code = ART_END;
  vpath[N_DOT_POINTS+1].x = 0.0;
  vpath[N_DOT_POINTS+1].y = 0.0;

}


static void gtk_canvas_dot_update(GtkCanvasItem *item,
				  double affine[6],
				  ArtSVP *clip_path, gint flags)
{
  GtkCanvasRE *re;
  GtkCanvasDot *dot;
  ArtVpath vpath[N_DOT_POINTS + N_DOT_POINTS + 3];
  ArtVpath *vpath2;
  double w_center_x, w_center_y, w_radius;
  extern GtkCanvasREClass *parent_class;
  /* Utility functions originally defined in gtk-canvas-rect-ellipse.c */
  static void set_gc_foreground(GdkGC*, gulong);
  static void set_stipple(GdkGC*, GdkBitmap**, GdkBitmap*, int);
  static void get_bounds(GtkCanvasDot *, int*, int*, int*, int*);

  re = GTK_CANVAS_RE(item);
  dot = GTK_CANVAS_DOT(item);
  
  /* Chain parent-class updates.  Fails at compile time becuase
     the class has no "update" member... */
  /* if (parent_class->update) 
     (*parent_class->update)(item, affine, clip_path, flags); */
  
  if(!item->canvas->aa) {
    set_gc_foreground(re->fill_gc, re->fill_pixel); 
    set_stipple(re->fill_gc, &re->fill_stipple, re->fill_stipple, TRUE);
  }
    
  if (item->canvas->aa) {
    /* Want to draw the circle in the canvas coordinates.  Transform
       the origin, then draw. */
    vpath[0].code = ART_MOVETO;
    vpath[0].x = dot->x;
    vpath[0].y = dot->y;
    
    vpath[1].code = ART_END;
    vpath[1].x = 0;
    vpath[1].y = 0;
    
    /* Transform the center of the dot to world coordinates. */
    vpath2 = art_vpath_affine_transform(vpath, affine);
    w_center_x = vpath2[0].x;
    w_center_y = vpath2[0].y;
    w_radius = (double)dot->size;
    art_free(vpath2);

    /* Overwrite vpath with the world-space path for the dot. */
    private_circle_gen(vpath, w_center_x, w_center_y, w_radius);
    
    gtk_canvas_item_update_svp_clip(item, &re->fill_svp, 
				    art_svp_from_vpath(vpath), clip_path);
  }
  else {
    int x0, x1, y0, y1;
    get_bounds(dot, &x0, &y0, &x1, &y1);
    gtk_canvas_update_bbox(item, x0, y0, x1, y1);
  }
}


static void gtk_canvas_dot_render(GtkCanvasItem *item,
				  GtkCanvasBuf *buffer)
{
  GtkCanvasRE *re;

  re = GTK_CANVAS_RE(item);

  if(re->fill_svp != 0) 
    gtk_canvas_render_svp(buffer, re->fill_svp, re->fill_color);
}


static void gtk_canvas_dot_draw(GtkCanvasItem *item,
				GdkDrawable *drawable,
				int x, int y, int width, int height)
{
  double i2w[6], w2c[6], i2c[6];
  ArtPoint i_center, c_center;
  
  GtkCanvasRE *re;
  GtkCanvasDot *dot;

  re = GTK_CANVAS_RE(item);
  dot = GTK_CANVAS_DOT(item);

  /* Retrieve affine transformations. */
  gtk_canvas_item_i2w_affine(item, i2w);
  gtk_canvas_w2c_affine(item->canvas, w2c);
  
  /* Build the item-to-canvas transformation. */
  art_affine_multiply(i2c, i2w, w2c);

  /* Use the item-to-canvas transformation to find the
     coordinates of the dot in canvas pixel coordinates. */
  i_center.x = dot->x;
  i_center.y = dot->y;
  art_affine_point(&c_center, &i_center, i2c);
  
  if (re->fill_stipple) 
    gtk_canvas_set_stipple_origin(item->canvas, re->fill_gc);

  gdk_draw_arc(drawable, re->fill_gc, TRUE, 
	       c_center.x-(dot->size)-x, c_center.y-(dot->size)-y,
	       2*(dot->size), 2*(dot->size),
	       0*64, 360*64);
}


/* Event-processing helper function -- reveals whether or not the
   point in question is inside or outside the dot. Points
   are passed in in item and canvas coords. */
static double gtk_canvas_dot_point(GtkCanvasItem *item,
				 double x, double y, int cx, int cy,
				 GtkCanvasItem **actual_item)
{
  GtkCanvasDot *dot;
  double dx, dy, w_radius;
  
  dot = GTK_CANVAS_DOT(item);
  *actual_item = item;
  
  dx = x-dot->x;
  dy = y-dot->y;
  w_radius = ((double)dot->size)/(item->canvas->pixels_per_unit);

  if ((dx*dx+dy*dy)<(w_radius*w_radius))
    return 0.0;
  else
    return dx*dx+dy*dy;
}


static void get_bounds (GtkCanvasDot *dot, int *px1, int *py1, 
			int *px2, int *py2)
{
  GtkCanvasItem *item;
  double x, y, w_size;
  int cx, cy;

  item = GTK_CANVAS_ITEM(dot);
  
  x = dot->x;
  y = dot->y;

  gtk_canvas_item_i2w (item, &x, &y);

  w_size = ((double)dot->size)/(item->canvas->pixels_per_unit);
  
  
  gtk_canvas_w2c (item->canvas, x, y, &cx, &cy);
  
  /* Bounding box is in canvas coordinates. */
  *px1 = cx-(dot->size)-1; /* Leftmost */
  *py1 = cy-(dot->size)-1; /* Graphically top, smallest y */
  *px2 = cx+(dot->size)+1;
  *py2 = cy+(dot->size)+1;
}




/******************************************************************/
/* Generic Gtk-object methods follow.                             */
/******************************************************************/

/* Local arguments.  Others are available in parent class. */
enum {
  ARG_0,  /* Dummy, arg_ids must be greater than zero. */
  ARG_X, 
  ARG_Y, 
  ARG_SIZE 
};


/* static void gtk_canvas_dot_base_init(void); */
static void gtk_canvas_dot_class_init(GtkCanvasDotClass*);
static void gtk_canvas_dot_init(GtkCanvasDot*);
GtkType gtk_canvas_dot_get_type();
static void gtk_canvas_dot_set_arg(GtkObject*, GtkArg*, guint);
static void gtk_canvas_dot_get_arg(GtkObject*, GtkArg*, guint);



/* Need to uninherit canvas-re stuff? */
/* static void gtk_canvas_dot_base_init(void) {} */

/* Set up the function pointers, register signals and 
   register object arguments. */
static void gtk_canvas_dot_class_init(GtkCanvasDotClass *klass) {
  GtkObjectClass *object_class;
  GtkCanvasItemClass *item_class;

  extern GtkCanvasREClass *parent_class;
  
  object_class = (GtkObjectClass*) klass;
  item_class = (GtkCanvasItemClass*) klass;

  parent_class = gtk_type_class (GTK_CANVAS_TYPE_CANVAS_RE);

  gtk_object_add_arg_type("GtkCanvasDot::x",
			  GTK_TYPE_DOUBLE,
			  GTK_ARG_READWRITE,
			  ARG_X);
  gtk_object_add_arg_type("GtkCanvasDot::y",
			  GTK_TYPE_DOUBLE,
			  GTK_ARG_READWRITE,
			  ARG_Y);
  gtk_object_add_arg_type("GtkCanvasDot::size",
			  GTK_TYPE_UINT,
			  GTK_ARG_READWRITE,
			  ARG_SIZE);
  
  object_class->set_arg = gtk_canvas_dot_set_arg;
  object_class->get_arg = gtk_canvas_dot_get_arg;

  item_class->update = gtk_canvas_dot_update;
  item_class->render = gtk_canvas_dot_render;
  item_class->draw   = gtk_canvas_dot_draw;
  item_class->point  = gtk_canvas_dot_point;
}

/* Set up the data members with reasonable defaults. */
static void gtk_canvas_dot_init(GtkCanvasDot *dot) {
  dot->x = 0.0;
  dot->y = 0.0;
  dot->size = 1;
}

GtkType gtk_canvas_dot_get_type(void) 
{
  static GtkType dot_type = 0;

  if (dot_type==0) {
    static const GtkTypeInfo dot_info = 
      { "GtkCanvasDot",
	sizeof(GtkCanvasDot),
	sizeof(GtkCanvasDotClass),
	(GtkClassInitFunc) gtk_canvas_dot_class_init,
	(GtkObjectInitFunc) gtk_canvas_dot_init,
	0, 0, /* Legacy reserved stuff */
	(GtkClassInitFunc) 0 /* gtk_canvas_dot_base_init */
      };
    dot_type = gtk_type_unique(GTK_CANVAS_TYPE_CANVAS_RE, &dot_info);
  }
  return dot_type;
}

static void gtk_canvas_dot_set_arg(GtkObject *object,
				   GtkArg *arg, 
				   guint arg_id) 
{
  GtkCanvasDot *dot;
  GtkCanvasItem *item;

  dot = GTK_CANVAS_DOT(object);
  item = GTK_CANVAS_ITEM(object);

  switch(arg_id) {
  case ARG_X:
    dot->x = GTK_VALUE_DOUBLE(*arg);
    gtk_canvas_item_request_update(item);
    break;
  case ARG_Y:
    dot->y = GTK_VALUE_DOUBLE(*arg);
    gtk_canvas_item_request_update(item);
    break;
  case ARG_SIZE:
    dot->size = GTK_VALUE_UINT(*arg);
    gtk_canvas_item_request_update(item);
    break;
  default:
    break;
  }
}


static void gtk_canvas_dot_get_arg(GtkObject *object,
				   GtkArg *arg,
				   guint arg_id) 
{
  GtkCanvasDot *dot;
  dot = GTK_CANVAS_DOT(object);

  switch(arg_id) {
  case ARG_X:
    GTK_VALUE_DOUBLE(*arg) = dot->x;
    break;
  case ARG_Y:
    GTK_VALUE_DOUBLE(*arg) = dot->y;
    break;
  case ARG_SIZE:
    GTK_VALUE_UINT(*arg) = dot->size;
    break;
  default:
    break;
  }
}


/* Utility functions cut-and-pasted out of gtk-canvas-rect-ellipse.c,
   because they're static and can't be linked-to. */

/* Sets the stipple pattern for the specified gc */
static void set_stipple (GdkGC *gc, GdkBitmap **internal_stipple, 
			 GdkBitmap *stipple, int reconfigure)
{
  if (*internal_stipple && !reconfigure)
    gdk_bitmap_unref (*internal_stipple);
  
  *internal_stipple = stipple;
  if (stipple && !reconfigure)
    gdk_bitmap_ref (stipple);
  
  if (gc) {
    if (stipple) {
      gdk_gc_set_stipple (gc, stipple);
      gdk_gc_set_fill (gc, GDK_STIPPLED);
    } else
      gdk_gc_set_fill (gc, GDK_SOLID);
  }
}

/* Convenience function to set a GC's foreground color to the
   specified pixel value */
static void set_gc_foreground (GdkGC *gc, gulong pixel)
{
  GdkColor c;
  
  if (!gc)
    return;
  
  c.pixel = pixel;
  gdk_gc_set_foreground (gc, &c);
}

