# -*- python -*-
# $RCSfile: labelledslider.py,v $
# $Revision: 1.7 $
# $Author: langer $
# $Date: 2004/08/13 18:27:39 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import gtk
from common import utils
from common import debug


class LabelledSlider:
    def __init__(self, value=None, vmin=0, vmax=1, step=0.01, callback=None):
        # "callback" is called when the user moves the slider.
        debug.mainthreadTest()
        self.gtk = gtk.GtkHBox()
        if value is None:
            value = vmin
        self.adjustment = gtk.GtkAdjustment(value=value,
                                            lower=vmin, upper=vmax,
                                            step_incr=step,
                                            page_incr=step)
        self.slider = gtk.GtkHScale(self.adjustment)
        self.gtk.add(self.slider)
        self.slider.set_draw_value(0)   # we'll display the value in self.entry
        self.adjustment.connect('value-changed', self.text_from_slider)

        self.callback = callback
        
        self.entry = gtk.GtkEntry()
        self.gtk.add(self.entry)
        font = self.entry.get_style().font
        width = max(font.width(`vmin`), font.width(`vmax`), 9*font.width('0'))
        self.entry.set_usize(width, -1)
        self.entry.connect('activate', self.slider_from_text)
        self.entry.connect('focus-out-event', self.entry_lost_focus)

        self.set_value(value)

    def set_digits(self, digits):
        "Sets number of digits after the decimal place."
        debug.mainthreadTest()
        self.slider.set_digits(digits)

    def set_value(self, value):
        debug.mainthreadTest()
        if value < self.adjustment.lower:
            self.adjustment.set_value(self.adjustment.lower)
        elif value > self.adjustment.upper:
            self.adjustment.set_value(self.adjustment.upper)
        else:
            self.adjustment.set_value(value)
        self.adjustment.value_changed()
    def get_value(self):
        debug.mainthreadTest()
        return self.adjustment.value
    def text_from_slider(self, obj):
        debug.mainthreadTest()
        self.entry.set_text("%-8g" % self.adjustment.value)
        self.entry.set_position(0)
        if self.callback:
            self.callback(self, self.adjustment.value)
    def slider_from_text(self, obj):
        debug.mainthreadTest()
        v0 = utils.OOFeval(self.entry.get_text())
        v = max(v0, self.adjustment.lower)
        v = min(v, self.adjustment.upper)
        self.adjustment.set_value(v)
        if v != v0:
            self.text_from_slider(obj)      # 
    def entry_lost_focus(self, obj, event):
        debug.fmsg()
        self.slider_from_text(obj)
