# -*- python -*-
# $RCSfile: progressbar_.py,v $
# $Revision: 1.24 $
# $Author: langer $
# $Date: 2004/09/21 19:23:56 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


## TODO LATER: Fix the following comment.

## TextProgressBar wraps the cProgressBar so that it can recieve non-numeric
## "types". It is the base class to the GtkProgressBar, which is defined
## elsewhere.

from SWIG.common import switchboard
from common.IO import threadmanager

_progressbar_options = [
    "continuous",
    "active",
    "discrete"
    ]

class ProgressBar :
    def __init__(self, type = "continuous", name = None):
        self.progressbar = getcProgressBar()
        if type in _progressbar_options: ## check validity
            self.type = type ## assign progressbar type
        else:
            raise AttributeError("progress bar option not valid")
        self.Worker = None
        self.name = name
        ## After bar is created, pass it to threadManager
        self.register_to_Thread_Manager()
        
    def __getattr__(self, attrname):
        return getattr(self.progressbar, attrname)
    
    
## These are the basic functions for a ProgressBar
    def get_name(self):
        return self.name
    
    def set_name(self, name):
        self.name = name
        
    def registerWorker(self, wrkr):
        self.Worker = wrkr
        
        
    def unregisterWorker(self):
        self.Worker = None

    def register_to_Thread_Manager(self):
        threadmanager.threadManager.newProgressBar(self)
        
    def get_type(self):
        return self.type
        
    def get_cProgressBar(self):
        return self.progressbar

    
#########################################################################
#########################################################################
## The following is the basic class for creating a progress bar
## representation. Each representation has different API for drawing
## the progress bar.
#########################################################################
#########################################################################
    
class ProgressBarRepresentation:
    def __init__(self, progressBar):
        self.progressbar = progressBar
        self.type = self.progressbar.get_type()
        self.name = self.progressbar.get_name()
        self.talk = 1
    def get_name(self):
        return self.progressbar.get_name()
    
        
#########################################################################
#########################################################################
## The following describes the progress bar representation for a text
## progress bar. In this case, the progress bar object returnd a string
## whose length and contents depends on the current state of the thread.
#########################################################################
#########################################################################
        
class TextProgressBar(ProgressBarRepresentation):
    def __init__(self, progressBar):
        ProgressBarRepresentation.__init__(self, progressBar)
        self.bar_length = 10
        self.window_width = 80
        self.bar_repr = None
        self.total_bar = " "
        self.direction = 1
        self.counter = 0

    def __getattr__(self, attrname):
        return getattr(self.progressbar, attrname)
        

    def bar_size(self):
        possible_sz =  self.window_width- len(self.get_message())- 5
        if possible_sz > 0:
            return possible_sz
        return 5
    
    def set_window_size(self, szz):
        self.window_width = szz
        
    ## functions to draw different types of progress bars
    def progress_active(self):
        actual_bar = ""
        self.bar_length = self.bar_size()
        for i in range(self.bar_length):
            if  i==counter:
                actual_bar += "*"
            else:
                actual_bar += " "
                
  
        if self.counter < self.bar_length and self.counter >=0 :
            self.counter  += self.direction
        else:
            self.direction *= -1
            self.counter  += self.direction
    
        return actual_bar

    def progress_incremental(self, progress):
        actual_bar = ""
        self.bar_length = self.bar_size()
        for i in range(self.bar_length):
            if i<=self.bar_length*progress:
                actual_bar += "*"
            else:
                actual_bar +=" "
        return actual_bar
    
    def buildprogressbar(self):
        ## Updates text progress bar.
        ## For activity mode, the passed value 
        ## is irrelevant. Valid values range 
        ## between zero and one.
        string_fraction = ""
        if self.type == "active":
            string_fraction = self.progress_active()
        if self.type == "continuous":
            string_fraction = self.progress_incremental(self.get_progress())
        self.total_bar = self.progressbar.get_message() + " |" + string_fraction + "| " + "\r"
        return self.total_bar



   

def set_ProgressBarRepresentation(pbar):
    ## argument is a ProgressBar.. This function is redefined if the
    ## GUI is loaded.
    return TextProgressBar(pbar)
    



