# -*- python -*-
# $RCSfile: analyzemenu.py,v $
# $Revision: 1.10 $
# $Author: reida $
# $Date: 2004/05/26 20:27:00 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import utils
from common.IO import automatic
from common.IO import destinationparam
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import output
from common.IO import parameter
from common.IO import reporter
from common.IO import whoville
from engine.IO import analyze
import engine.mesh
import sys

ops_menu = oofmenu.OOFMenuItem("Analyze", secret=1)
mainmenu.OOF.Mesh.addItem(ops_menu)


mesh_parameter = whoville.WhoParameter(
    "mesh", engine.mesh.meshes,
    tip="The mesh from which the data should be taken.")
output_parameter = output.ScalarOutputParameter(
    "output",tip="The data to write.")
destination_parameter = destinationparam.DestinationParameter(
    "destination", value=automatic.automatic, default=automatic.automatic,
    tip="File where data should be written, or 'automatic' for the message window.")



def _ops_callback(menuitem, mesh, output, destination, **kwargs):
    callable = menuitem.data()
    mesh_obj = engine.mesh.meshes[mesh]

    # "destination" is a file name, or "automatic" -- use
    # destinationparam.get_file to convert it to a file handle.
    outfile = destinationparam.get_file(destination)

    callable(mesh_obj, output, outfile)
    outfile.flush()




# Create a menu item for each registered object in the DataOperation
# class.
def makemenu(menu_base):
    menu_base.clearSubMenu()
    for r in analyze.DataOperation.registry:
        new_item = oofmenu.OOFMenuItem(
            utils.space2underscore(r.name()),
            callback = _ops_callback,
            params = [mesh_parameter,
                      output_parameter,
                      destination_parameter] + r.params )
        new_item.data = r
        menu_base.addItem(new_item)

makemenu(ops_menu)

switchboard.requestCallback(analyze.DataOperation, makemenu, ops_menu)
