# -*- python -*-
# $RCSfile: skeletonbdytoolbox.py,v $
# $Revision: 1.16 $
# $Author: reida $
# $Date: 2004/04/29 20:23:43 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# There is a toolbox for viewing the boundaries associated with
# a skeleton -- it has commands allowing you to view only the
# edgeboundaries, only the nodeboundaries, both, or individual
# boundaries selected by name.  It is not intended to allow
# you to modify boundaries or apply boundary conditions -- boundaries
# are modified by making a new element/segment/node group and
# converting it to a boundary, and application of boundary
# conditions happens in the mesh, where the physics lives.

from common import toolbox
from SWIG.common import switchboard
from engine import skeletoncontext


class SkeletonBoundaryToolbox(toolbox.Toolbox):
    def __init__(self, gfxwindow):
        self.whoset = ('Skeleton',)
        toolbox.Toolbox.__init__(self, name="Boundaries", gfxwindow=gfxwindow)
        self.skeletoncontext = None
        self.sb_callbacks = [
            switchboard.requestCallback("new boundaries", self.new_boundaries),
            switchboard.requestCallback("new boundary configuration",
                                        self.new_config),
            switchboard.requestCallback((self.gfxwindow(), "layers changed"),
                                        self.newLayers ),
            ]

    def close(self):
        for s in self.sb_callbacks:
            switchboard.removeCallback(s)
        
    # Find the top-most skeleton and make it current. 
    def activate(self):
        self.skeletoncontext = apply(self.gfxwindow().topwho, self.whoset)
        if self.skeletoncontext:
            bdyset = (self.skeletoncontext.getObject().edgeboundaries.keys(),
                      self.skeletoncontext.getObject().pointboundaries.keys() )
            switchboard.notify((self.gfxwindow(), 'new boundary set'), bdyset)

    def newLayers(self):
        self.activate()

    # "new boundaries" is called when boundaries are added to or
    # removed from the skeletoncontext.
    def new_boundaries(self, skeletoncontext):
        if self.skeletoncontext==skeletoncontext:
            bdyset = (self.skeletoncontext.getObject().edgeboundaries.keys(),
                      self.skeletoncontext.getObject().pointboundaries.keys() )
            switchboard.notify((self.gfxwindow(), 'new boundary set'), bdyset)

    # The "new boundary configuration" callback is for when the
    # boundaries have changed (had edges added, etc.) but the
    # selection state has not changed.
    def new_config(self, skeletoncontext):
        if self.skeletoncontext==skeletoncontext:
            switchboard.notify((self.gfxwindow(), "new boundary config"))
            

toolbox.registerToolboxClass(SkeletonBoundaryToolbox, ordering=1.9)
