// -*- C++ -*-
// $RCSfile: boundarycond.h,v $
// $Revision: 1.12 $
// $Author: langer $
// $Date: 2004/09/02 18:24:12 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef BOUNDARYCOND_H
#define BOUNDARYCOND_H

#include <Python.h>
#include "common/tostring.h"
#include "sparselink.h"

class EdgeSet;
class Field;
class Flux;
class Equation;
class FuncNode;
class FEMesh;



// FloatBCApp is the "applicator" for the floating boundary
// condition class, which is in Python.  The calling semantics
// are that, when you get to this level, you should have
// a DOF and a nodalequation in-hand, along with the
// mapping lists. 

// For now, instancing the FloatBCApp sets things up by
// setting firstcall to 1.

class FloatBCApp {
private:
  std::vector<double> profile_vector;      // Profile-function offsets.
  std::vector<int> rhssubtractor_index;     // Column-number map
public:
  FloatBCApp() {}
  ~FloatBCApp() {}
  int getdofindex(FEMesh*, FuncNode*, Field*, int);
  int geteqnindex(FEMesh*, FuncNode*, Equation*, int);
  void editmap(FEMesh *, double, FuncNode *, Field *, int, Equation *, 
	       int, int, int, double);
  void profile_rhs(FEMesh *);
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// "Applicator" for the flux boundary condition.  

class NeumannBCApp {
private:
  FEMesh *mesh;
  Flux *flux;
  EdgeSet *bdy;
public:
  //  NeumannBCApp(FEMesh *m, Flux *f, Boundary *b) : mesh(m), flx(f), bdy(b) {}
  NeumannBCApp(FEMesh *m, Flux *f, EdgeSet *b) : mesh(m), flux(f), bdy(b) {}
  ~NeumannBCApp() {}
  void integrate(PyObject *, PyObject *, bool normal);
};


// ForceBC is the node-based twin of NeumannBC.  Flux's make sense to
// integrate, forces are associated with equations and make sense for
// direct application at nodes.

void applyForceBC(FEMesh*, Equation*, FuncNode*, int eqnindex, double value);
  
#endif

