# -*- python -*-
# $RCSfile: problem.py,v $
# $Revision: 1.21 $
# $Author: langer $
# $Date: 2004/07/21 18:36:44 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import sys, types

from SWIG.common import switchboard
from SWIG.common.space import spacedim
from SWIG.engine import field, flux, equation
from common import debug
from common import utils

def define(obj):
    utils.OOFdefine(obj.name(), obj)
    return obj

def defineField(fld):
    define(fld)
    out_of_plane = fld.out_of_plane();
    define(out_of_plane)
    # "new field" is sent here, instead of from the Field constructor,
    # because it must be called *after* the field is defined in the
    # OOF namespace.
    switchboard.notify("new field")
    return fld

def defineFlux(flx):
    define(flx)
    switchboard.notify("new flux")
    return flx

def defineEquation(eqn):
    define(eqn)
    switchboard.notify("new equation")
    return eqn

####################

# Define a field.  This creates an object named 'Temperature' in the
# OOF namespace.
Temperature = defineField(field.ScalarField('Temperature'))
# Define a flux
Heat_Flux = defineFlux(flux.VectorFlux('Heat_Flux'))
# And equations
defineEquation(equation.DivergenceEquation('Heat_Eqn', Heat_Flux, 1))
defineEquation(equation.PlaneFluxEquation('Plane_Heat_Flux', Heat_Flux, 1))

Displacement = defineField(field.TwoVectorField('Displacement'))
Stress = defineFlux(flux.SymmetricTensorFlux('Stress'))
defineEquation(equation.DivergenceEquation('Force_Balance', Stress, spacedim))
defineEquation(equation.PlaneFluxEquation('Plane_Stress', Stress, 3))
