# -*- python -*-
# $RCSfile: profilefunction.py,v $
# $Revision: 1.6 $
# $Author: reida $
# $Date: 2004/07/07 13:32:41 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import strfunction
from common import utils
from common.IO import parameter
import struct


# String-based function object used by boundary profiles -- user types
# in a string, and one of these objects gets built.  Profiles'
# __call__ methods provide the relevant arguments.

class ProfileFunction(strfunction.StrFunction):
    def __init__(self, funcstr):
        # x and y are the current location and is guaranteed to be
        # provided. All others are provided if applicable.  i is the
        # node index, s is the arc length, alpha is the fractional arc
        # length, and nx and ny are the local normal.
        strfunction.StrFunction.__init__(
            self, 'x, y, i, s, alpha, nx, ny', funcstr)
    def __repr__(self):
        return "ProfileFunction('%s')" % self.funcstr

class ProfileFunctionParameter(parameter.ParameterBase):
    def __init__(self, name, value=None, default=None, tip=None):
        self.checker = parameter.TypeChecker(ProfileFunction)
        parameter.ParameterBase.__init__(self, name,
                                         value=value,
                                         default=default, tip=tip)
    def binaryRepr(self, datafile, value):
        length = len(value.funcstr)
        return struct.pack('>i', length) + value.funcstr
    def binaryRead(self, parser):
        b = parser.getBytes(struct.calcsize('>i'))
        (length,) = struct.unpack('>i', b)
        return ProfileFunction(parser.getBytes(length))
    

utils.OOFdefine('ProfileFunction', ProfileFunction)

