// -*- C++ -*-
// $RCSfile: massdensity.C,v $
// $Revision: 1.16 $
// $Author: langer $
// $Date: 2004/05/05 17:56:58 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "common/coord.h"
#include "massdensity.h"
#include "engine/element.h"
#include "engine/elementnodeiterator.h"
#include "engine/field.h"
#include "engine/flux.h"
#include "engine/indextypes.h"
#include "engine/material.h"
#include "engine/femesh.h"
#include "common/space.h"
#include "common/trace.h"
#include "common/ooferror.h"
using namespace std;

MassDensityProp::MassDensityProp(PyObject *reg, const string &nm, double rho)
  : Property(nm,reg),
    density(rho)
{
  displacement = dynamic_cast<TwoVectorField*>(Field::getField("Displacement"));
  stress_flux = dynamic_cast<SymmetricTensorFlux*>(Flux::getFlux("Stress"));
}

void MassDensityProp::precompute() {
}

int MassDensityProp::K_integration_order(int degree) const {
  return degree;
}

void MassDensityProp::fluxrhs(const FEMesh *mesh, const Element *element,
			      const Flux *flux, FluxData *fluxdata,
			      const MasterPosition &x) const 
{
  // For mass density, only need to set the appropriate vector,
  // it's the same at every point. 

  if(*flux != *stress_flux) {
    throw ErrProgrammingError("Unexpected flux", __FILE__, __LINE__);
  }

  Trace("MassDensityProp::fluxrhs");

  // Arbitrarily select the -y direction as "down".
  // In this case, the components are obvious, don't need to select them.
  for(int i=0;i<stress_flux->divergence_dim();i++) {
    if( i==(stress_flux->divergence_dim()-1) ) {
      fluxdata->rhs_element(i) += -density;
    }
  }
}

