# -*- python -*-
# $RCSfile: vigilante.py,v $
# $Revision: 1.3 $
# $Author: langer $
# $Date: 2004/07/27 14:36:28 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import debug
from common import primitives
from common import registeredclass
from common.IO import parameter
from common.IO import reporter
from engine import skeletonmodifier
import random

class FixIllegal(skeletonmodifier.SkeletonModifier):
    def apply(self, oldskeleton, context):
        skel = oldskeleton.properCopy()
        suckers = skel.illegalElements()
        random.shuffle(suckers)
        self.repented = {}

        # No new illegal elements should be added in the process.
        # Thus, the routine stops if len(self.repented) == len(suckers) or
        # it goes into an infinite loop.

        max = 222
        count = 0
        while len(self.repented) != len(suckers) and count < max:
            count += 1
            self.smoothIllegalElements(skel, suckers)

        if count < max:  # normal exit
            reporter.report("%d illegal element%s removed." % (len(self.repented), "s"*(len(self.repented)!=1)))

        skel.cleanUp()
        skel.checkIllegality()
        return skel

    def smoothIllegalElements(self, skel, suckers):
        for element in suckers:
            if self.repented.has_key(element):
                continue
            node_indices = range(element.nnodes())
            random.shuffle(node_indices)
            for i in node_indices:
                node = element.nodes[i]
                if not (element.getRealAngle(i) > 0.0):  # negative angle
                    self.smoothTheNode(skel, node, picky=random.choice([0,1]))
    
    def smoothTheNode(self, skel, node, picky):
        # how many illegal elements?
        guilty = []
        for e in node.neighborElements():
            if e.illegal():
                guilty.append(e)

        # moving a node to its avaerage position w.r.t. its neighbor nodes
        avgPos = primitives.Point(0.,0.)
        nbrNodes = node.neighborNodes(skel)
        nnodes = len(nbrNodes)
        if picky:
            illegal_nodes = 0
        for n in nbrNodes:
            # If "picky" flag is up, don't bother with illegal node.
            if picky and n.illegal():
                illegal_nodes += 1
                continue
            avgPos += n.position()
        if picky:
            nnodes -= illegal_nodes
        if nnodes == 0:
            return
        avgPos /= nnodes
        skel.moveNodeTo(node, avgPos)
        
        # see if the move removes illegal element
        still_guilty = []
        for e in node.neighborElements():
            if e.illegal():
                still_guilty.append(e)

        if len(still_guilty) >= len(guilty):  # no improvement
            skel.moveNodeBack(node)
        else:
            for e in node.neighborElements():
                if e in guilty and not (e in still_guilty):
                    self.repented[e] = 1
            
#########################

registeredclass.Registration(
    'Fix Illegal Elements', skeletonmodifier.SkeletonModifier,
    FixIllegal,
    ordering=99,
    ok_illegal = 1,  # It can deal with illegal skeletons 
    tip = 'Tools to clean up illegal elements.')
